if (!window.PluginUtil) {
    var PluginUtil = {
        register: function(namespace, pluginName, rawPlugin) {
            for (var methodName in rawPlugin) {
                if (!rawPlugin.hasOwnProperty(methodName) || (typeof rawPlugin[methodName] != 'function'))
                    continue;
                window[namespace] = window[namespace] || {};
                if (!window[namespace][pluginName]) {
                    window[namespace][pluginName] = {};
                }

                var pluginProxy = window[namespace][pluginName];
                pluginProxy[methodName] = (function(methodName) {
                    return function() {
                        if (window.plus && window.plus.isReady) {
                            rawPlugin[methodName].apply(window, arguments);
                        } else {
                            document.addEventListener("plusready",  function() {
                                rawPlugin[methodName].apply(window, arguments);
                            }, true);
                        }
                    }
                })(methodName);
            }
        }
    };
}


(function() {


    //插件名称
    var PLUGIN_NAME = "CallLogPlugin";
    //定义JS插件
    var CallLogPlugin = {

        /**
         * 获取n天通话记录的总条数
         *
         * days 统计的天数，int类型
         * countPerson true按联系人的号码统计（重复号码计为1条），boolean类型
         * successCallback 成功回调
         * errorCallback 失败回调
         */
        getCallLogCount : function (days, countPerson, successCallback, errorCallback) {
            var success = typeof successCallback !== 'function' ? null : function(data) {
                successCallback(data);
            };
            var fail = typeof errorCallback !== 'function' ? null : function(args) {
                errorCallback(args);
            };
            var callbackId = window.plus.bridge.callbackId(success, fail);
            var args = [callbackId, days, countPerson];
            return window.plus.bridge.exec(PLUGIN_NAME, "getCallLogCount", args);
        },

        /**
         * 根据号码查找它的通话记录列表
         * phoneNo 电话号码，String类型
         * successCallback 成功回调
         * errorCallback 失败回调
         * 返回通话记录列表数组，单个通话记录对象：{phoneNo:xxx, phoneName:xxx, callTime:xxx, callTimeFormat:xxx}
         */
        getCallTimeByNo : function (phoneNo, successCallback, errorCallback) {
            var success = typeof successCallback !== 'function' ? null : function(data) {
                successCallback(data);
            };
            var fail = typeof errorCallback !== 'function' ? null : function(args) {
                errorCallback(args);
            };
            var callbackId = window.plus.bridge.callbackId(success, fail);
            var args = [callbackId, phoneNo];
            return window.plus.bridge.exec(PLUGIN_NAME, "getCallTimeByNo", args);
        },

        /**
         * 获取n天通话记录列表
         * days 获取通话记录的天数
         * successCallback 成功回调
         * errorCallback 失败回调
         * 返回通话记录列表数组，单个通话记录对象：{phoneNo:xxx, phoneName:xxx, callTime:xxx, callTimeFormat:xxx}
         */
        getCallLogs : function (days, successCallback, errorCallback) {
            var success = typeof successCallback !== 'function' ? null : function(data) {
                successCallback(data);
            };
            var fail = typeof errorCallback !== 'function' ? null : function(args) {
                errorCallback(args);
            };
            var callbackId = window.plus.bridge.callbackId(success, fail);
            var args = [callbackId, days];
            return window.plus.bridge.execSync(PLUGIN_NAME, "getCallLogs", args);
        },

    };

    window.PluginUtil.register("muna", PLUGIN_NAME, CallLogPlugin);
})();

