if (!window.PluginUtil) {
    var PluginUtil = {
        register: function(namespace, pluginName, rawPlugin) {
            for (var methodName in rawPlugin) {
                if (!rawPlugin.hasOwnProperty(methodName) || (typeof rawPlugin[methodName] != 'function'))
                    continue;
                window[namespace] = window[namespace] || {};
                if (!window[namespace][pluginName]) {
                    window[namespace][pluginName] = {};
                }

                var pluginProxy = window[namespace][pluginName];
                pluginProxy[methodName] = (function(methodName) {
                    return function() {
                        if (window.plus && window.plus.isReady) {
                            rawPlugin[methodName].apply(window, arguments);
                        } else {
                            document.addEventListener("plusready",  function() {
                                rawPlugin[methodName].apply(window, arguments);
                            }, true);
                        }
                    }
                })(methodName);
            }
        }
    };
}


(function() {


    //插件名称
    var PLUGIN_NAME = "ContactPlugin";
    //定义JS插件
    var ContactPlugin = {

        /**
         * 选择联系人
         *
         * successCallback 成功回调
         * errorCallback 失败回调
         */
        selectContact : function (successCallback, errorCallback) {
            var success = typeof successCallback !== 'function' ? null : function(data) {
                successCallback(data);
            };
            var fail = typeof errorCallback !== 'function' ? null : function(args) {
                errorCallback(args);
            };
            var callbackId = window.plus.bridge.callbackId(success, fail);
            var args = [callbackId];
            return window.plus.bridge.exec(PLUGIN_NAME, "selectContact", args);
        },

        /**
         * 根据号码查找联系人姓名,异步方法
         * 如果查找不到，返回null
         * phoneNo 要查询的号码，String类型
         * successCallback 成功回调
         * errorCallback 失败回调
         */
        getNameByNo : function (phoneNo, successCallback, errorCallback) {
            var success = typeof successCallback !== 'function' ? null : function(data) {
                successCallback(data);
            };
            var fail = typeof errorCallback !== 'function' ? null : function(args) {
                errorCallback(args);
            };
            var callbackId = window.plus.bridge.callbackId(success, fail);
            var args = [callbackId, phoneNo];
            return window.plus.bridge.exec(PLUGIN_NAME, "getNameByNo", args);
        },

        /**
         * 获取联系人总条数,异步方法
         * successCallback 成功回调
         * errorCallback 失败回调
         */
        getContactCount : function (successCallback, errorCallback) {
            var success = typeof successCallback !== 'function' ? null : function(data) {
                successCallback(data);
            };
            var fail = typeof errorCallback !== 'function' ? null : function(args) {
                errorCallback(args);
            };
            var callbackId = window.plus.bridge.callbackId(success, fail);
            var args = [callbackId];
            return window.plus.bridge.execSync(PLUGIN_NAME, "getContactCount", args);
        },

        /**
         * 根据号码查找联系人姓名,异步方法
         * 返回联系人姓名数组，[name1, name2, ...]
         * phoneNo 要查询的号码，String类型
         * successCallback 成功回调
         * errorCallback 失败回调
         */
        getAllNameByNo : function (phoneNo, successCallback, errorCallback) {
            var success = typeof successCallback !== 'function' ? null : function(data) {
                successCallback(data);
            };
            var fail = typeof errorCallback !== 'function' ? null : function(args) {
                errorCallback(args);
            };
            var callbackId = window.plus.bridge.callbackId(success, fail);
            var args = [callbackId, phoneNo];
            return window.plus.bridge.execSync(PLUGIN_NAME, "getAllNameByNo", args);
        },

        /**
         * 根据联系人姓名查找所有号码，异步方法
         * 返回联系人电话号码数组 [num1, num2, ...]
         * name 要查询的联系人名称，String类型
         * successCallback 成功回调
         * errorCallback 失败回调
         */
        getAllNoByName : function (name, successCallback, errorCallback) {
            var success = typeof successCallback !== 'function' ? null : function(data) {
                successCallback(data);
            };
            var fail = typeof errorCallback !== 'function' ? null : function(args) {
                errorCallback(args);
            };
            var callbackId = window.plus.bridge.callbackId(success, fail);
            var args = [callbackId, name];
            return window.plus.bridge.execSync(PLUGIN_NAME, "getAllNoByName", args);
        },


        /**
         * 根据联系人姓名查找所有号码及分组信息，异步方法
         * 返回联系人查找到的号码及分组信息数组，分组对象为 {phoneNo:xxx , phoneName:xxx, groups:xxx}
         * name 要查询的联系人名称，String类型
         * successCallback 成功回调
         * errorCallback 失败回调
         */
        getGroupInfoByName : function (name, successCallback, errorCallback) {
            var success = typeof successCallback !== 'function' ? null : function(data) {
                successCallback(data);
            };
            var fail = typeof errorCallback !== 'function' ? null : function(args) {
                errorCallback(args);
            };
            var callbackId = window.plus.bridge.callbackId(success, fail);
            var args = [callbackId, name];
            return window.plus.bridge.execSync(PLUGIN_NAME, "getGroupInfoByName", args);
        },

        /**
         * 根据联系人号码查找所有联系人姓名及分组信息，异步方法
         * 返回联系人查找到的号码及分组信息数组，分组对象为 {phoneNo:xxx , phoneName:xxx, groups:xxx}
         * phoneNo 要查询的手机号码，String类型
         * successCallback 成功回调
         * errorCallback 失败回调
         */
        getGroupInfoByNo : function (phoneNo, successCallback, errorCallback) {
            var success = typeof successCallback !== 'function' ? null : function(data) {
                successCallback(data);
            };
            var fail = typeof errorCallback !== 'function' ? null : function(args) {
                errorCallback(args);
            };
            var callbackId = window.plus.bridge.callbackId(success, fail);
            var args = [callbackId, phoneNo];
            return window.plus.bridge.execSync(PLUGIN_NAME, "getGroupInfoByNo", args);
        },

        /**
         * 获取整个联系人列表
         * 返回联系人列表数组，单个对象为 {phoneNo:xxx , phoneName:xxx, groups:xxx}
         * successCallback 成功回调
         * errorCallback 失败回调
         */
        getAllContacts : function (successCallback, errorCallback) {
            var success = typeof successCallback !== 'function' ? null : function(data) {
                successCallback(data);
            };
            var fail = typeof errorCallback !== 'function' ? null : function(args) {
                errorCallback(args);
            };
            var callbackId = window.plus.bridge.callbackId(success, fail);
            var args = [callbackId];
            return window.plus.bridge.execSync(PLUGIN_NAME, "getAllContacts", args);
        },
    };

    window.PluginUtil.register("muna", PLUGIN_NAME, ContactPlugin);
})();

