/*
 * @Author: xiexiongbiao
 * @desc:   muna插件
 * @changelog:   
    2016-4-29 修复js报错时候的错误处理
 */

var PLUGIN_NAME = "HttpRequestPlugin";

window.muna = window.muna || {};

function isArray(arr) {
  return Object.prototype.toString.apply(arr) === '[object Array]';
}

function extend() {
  var argLen = arguments.length;
  var target = arguments[0];

  if (argLen < 2) {
    return target;
  }

  for (var i = 1; i < argLen; i++) {
    extendSimple(target, arguments[i]);
  }
  return target;
}

function extendSimple(target, obj) {
  for (var key in obj) {
    if (!obj.hasOwnProperty(key)) continue;
    target[key] = obj[key];
  }
}

function trigger(elements, eventName, eventData) {
  var events = eventName.split(' ');
  for (var i = 0; i < events.length; i++) {
    var evtName = events[i];

    if (elements.legnth) {
      for (var j = 0; j < elements.length; j++) {
        triggerEvent(elements[i], evtName, eventData);
      }
    } else {
      triggerEvent(elements, evtName, eventData);
    }
  }
}

function triggerEvent(element, eventName, eventData) {
  var evt;
  try {
    evt = new CustomEvent(eventName, {detail: eventData, bubbles: true, cancelable: true});
  } catch(e) {
    evt = document.createEvent('Event');
    evt.initEvent(eventName, true, true);
    evt.detail = eventData;
  }
  element.dispatchEvent(evt);
}

function serializeObject(obj, parents) {
    if (typeof obj === 'string') return obj;
    var resultArray = [];
    var separator = '&';
    parents = parents || [];
    var newParents;
    function var_name(name) {
        if (parents.length > 0) {
            var _parents = '';
            for (var j = 0; j < parents.length; j++) {
                if (j === 0) _parents += parents[j];
                else _parents += '[' + encodeURIComponent(parents[j]) + ']';
            }
            return _parents + '[' + encodeURIComponent(name) + ']';
        }
        else {
            return encodeURIComponent(name);
        }
    }
    function var_value(value) {
        return encodeURIComponent(value);
    }
    for (var prop in obj) {
        if (obj.hasOwnProperty(prop)) {
            var toPush;
            if (isArray(obj[prop])) {
                toPush = [];
                for (var i = 0; i < obj[prop].length; i ++) {
                    if (!isArray(obj[prop][i]) && typeof obj[prop][i] === 'object') {
                        newParents = parents.slice();
                        newParents.push(prop);
                        newParents.push(i + '');
                        toPush.push(serializeObject(obj[prop][i], newParents));
                    }
                    else {
                        toPush.push(var_name(prop) + '[]=' + var_value(obj[prop][i]));
                    }
                    
                }
                if (toPush.length > 0) resultArray.push(toPush.join(separator));
            }
            else if (typeof obj[prop] === 'object') {
                // Object, convert to named array
                newParents = parents.slice();
                newParents.push(prop);
                toPush = serializeObject(obj[prop], newParents);
                if (toPush !== '') resultArray.push(toPush);
            }
            else if (typeof obj[prop] !== 'undefined' && obj[prop] !== '') {
                // Should be string or plain value
                resultArray.push(var_name(prop) + '=' + var_value(obj[prop]));
            }
        }
    }
    return resultArray.join(separator);
};

// Global Ajax Setup
var globalAjaxOptions = {};

muna.ajaxSetup = function (options) {
  if (options.type) options.method = options.type;
  extend(globalAjaxOptions, options);
};

function XHREmulator() {
  this.readyState = 0;
  this.responseText = "";
  this.responseType = "";
  this.responseXML = null;
  this.status = 0;
  this.statusText = "";
}

XHREmulator.prototype = {
  abort: function() {
    // console.warn('This method is not supported by native APP.');
  },

  getAllResponseHeaders: function() {
    // console.warn('This method is not supported by native APP.');
  },

  send: function() {
    // console.warn('This method is not supported by native APP.');
  },

  setRequestHeader: function() {
    // console.warn('This method is not supported by native APP.');
  }
};

var _jsonpRequests = 0;

muna.ajax = function(options) {
  var defaults = {
      method: 'GET',
      data: null,
      async: true,
      cache: true,
      user: '',
      password: '',
      headers: {},
      xhrFields: {},
      statusCode: {},
      processData: true,
      dataType: 'text',
      contentType: 'application/x-www-form-urlencoded',
      timeout: 0
  };
  var callbacks = ['beforeSend', 'error', 'complete', 'success', 'statusCode'];

  //For jQuery guys
  if (options.type) options.method = options.type;

  // Fill relative urls
  if (/^\/[^\/]/.test(options.url)) {
      var host = window.location.host;
      var protocol = window.location.protocol;
      options.url = protocol + '//' + host + options.url;
  }

  // Merge global and defaults
  for (var key in globalAjaxOptions) {
    if (!globalAjaxOptions.hasOwnProperty(key)) continue;
    var value = globalAjaxOptions[key];
    if (callbacks.indexOf(key) < 0) defaults[key] = value;
  }

  // Function to run XHR callbacks and events
  function fireAjaxCallback (eventName, eventData, callbackName) {
      var a = arguments;
      if (eventName) trigger(document, eventName, eventData);
      if (callbackName) {
          // Global callback
          if (callbackName in globalAjaxOptions) globalAjaxOptions[callbackName](a[3], a[4], a[5], a[6]);
          // Options callback
          if (options[callbackName]) options[callbackName](a[3], a[4], a[5], a[6]);
      }
  }

  options = extend({}, defaults, options);

  // Default URL
  if (!options.url) {
      options.url = window.location.toString();
  }
  // Parameters Prefix
  var paramsPrefix = options.url.indexOf('?') >= 0 ? '&' : '?';

  // UC method
  var _method = options.method.toUpperCase();
  // Data to modify GET URL
  if ((_method === 'GET' || _method === 'HEAD' || _method === 'OPTIONS' || _method === 'DELETE') && options.data) {
      var stringData;
      if (typeof options.data === 'string') {
          // Should be key=value string
          if (options.data.indexOf('?') >= 0) stringData = options.data.split('?')[1];
          else stringData = options.data;
      }
      else {
          // Should be key=value object
          stringData = serializeObject(options.data);
      }
      if (stringData.length) {
          options.url += paramsPrefix + stringData;
          if (paramsPrefix === '?') paramsPrefix = '&';
      }
  }

  // JSONP
  if (options.dataType === 'json' && options.url.indexOf('callback=') >= 0) {
  
      var callbackName = 'f7jsonp_' + Date.now() + (_jsonpRequests++);
      var abortTimeout;
      var callbackSplit = options.url.split('callback=');
      var requestUrl = callbackSplit[0] + 'callback=' + callbackName;
      if (callbackSplit[1].indexOf('&') >= 0) {
          var addVars = callbackSplit[1].split('&').filter(function (el) { return el.indexOf('=') > 0; }).join('&');
          if (addVars.length > 0) requestUrl += '&' + addVars;
      }
  
      // Create script
      var script = document.createElement('script');
      script.type = 'text/javascript';
      script.onerror = function() {
          clearTimeout(abortTimeout);
          fireAjaxCallback(undefined, undefined, 'error', null, 'scripterror');
      };
      script.src = requestUrl;
  
      // Handler
      window[callbackName] = function (data) {
          clearTimeout(abortTimeout);
          fireAjaxCallback(undefined, undefined, 'success', data);
          script.parentNode.removeChild(script);
          script = null;
          delete window[callbackName];
      };
      document.querySelector('head').appendChild(script);
  
      if (options.timeout > 0) {
          abortTimeout = setTimeout(function () {
              script.parentNode.removeChild(script);
              script = null;
              fireAjaxCallback(undefined, undefined, 'error', null, 'timeout');
          }, options.timeout);
      }
  
      return;
  }

  // Cache for GET/HEAD requests
  if (_method === 'GET' || _method === 'HEAD' || _method === 'OPTIONS' || _method === 'DELETE') {
      if (options.cache === false) {
          options.url += (paramsPrefix + '_nocache=' + Date.now());
      }
  }

  // Create XHR
  var xhr = new XHREmulator();
  
  // Save Request URL
  xhr.requestUrl = options.url;
  xhr.requestParameters = options;

  // Create POST Data
  var postData = null;
  
  if ((_method === 'POST' || _method === 'PUT' || _method === 'PATCH') && options.data) {
      /*if (options.processData) {
          var postDataInstances = [ArrayBuffer, Blob, Document, FormData];
          // Post Data
          if (postDataInstances.indexOf(options.data.constructor) >= 0) {
              postData = options.data;
          }
          else {
              // POST Headers
              var boundary = '---------------------------' + Date.now().toString(16);
  
              if (options.contentType === 'multipart\/form-data') {
                  xhr.setRequestHeader('Content-Type', 'multipart\/form-data; boundary=' + boundary);
              }
              else {
                  xhr.setRequestHeader('Content-Type', options.contentType);
              }
              postData = '';
              var _data = serializeObject(options.data);
              if (options.contentType === 'multipart\/form-data') {
                  boundary = '---------------------------' + Date.now().toString(16);
                  _data = _data.split('&');
                  var _newData = [];
                  for (var i = 0; i < _data.length; i++) {
                      _newData.push('Content-Disposition: form-data; name="' + _data[i].split('=')[0] + '"\r\n\r\n' + _data[i].split('=')[1] + '\r\n');
                  }
                  postData = '--' + boundary + '\r\n' + _newData.join('--' + boundary + '\r\n') + '--' + boundary + '--\r\n';
              }
              else {
                  postData = options.contentType === 'application/x-www-form-urlencoded' ? _data : _data.replace(/&/g, '\r\n');
              }
          }
      }
      else {
          postData = options.data;
      }*/

      // App does not support processed data, just send a js object.
      postData = options.data;
  }

  // Additional headers
  /*if (options.headers) {
      $.each(options.headers, function (headerName, headerCallback) {
          xhr.setRequestHeader(headerName, headerCallback);
      });
  }*/

  var success = function(data) {
      xhr.responseText = data.response;
      xhr.status = data.code;
      xhr.statusText = "success";

      if ((xhr.status >= 200 && xhr.status < 300) || xhr.status === 0) {
          var responseData;
          var parseSuccess = false;
          if (options.dataType === 'json') {
              try {
                  responseData = JSON.parse(xhr.responseText);
                  parseSuccess = true;
              }
              catch (err) {
                  fireAjaxCallback('ajaxError', {xhr: xhr, parseerror: true}, 'error', xhr, 'parseerror');
              }

              if (parseSuccess) {
                  fireAjaxCallback('ajaxSuccess', {xhr: xhr}, 'success', responseData, xhr.status, xhr);
              }
          }
          else {
              responseData = xhr.responseText;
              fireAjaxCallback('ajaxSuccess', {xhr: xhr}, 'success', responseData, xhr.status, xhr);
          }
      }
      else {
          fireAjaxCallback('ajaxError', {xhr: xhr}, 'error', xhr, xhr.status);
      }

      if (options.statusCode) {
          if (globalAjaxOptions.statusCode && globalAjaxOptions.statusCode[xhr.status]) globalAjaxOptions.statusCode[xhr.status](xhr);
          if (options.statusCode[xhr.status]) options.statusCode[xhr.status](xhr);
      }
      fireAjaxCallback('ajaxComplete', {xhr: xhr}, 'complete', xhr, xhr.status);
  };

  var fail = function(data) {
    xhr.responseText = data.response;
    xhr.status = data.errCode;
    xhr.statusText = data.errMsg;

    fireAjaxCallback('ajaxError', {xhr: xhr}, 'error', xhr, xhr.status);

    // is this needed?
    fireAjaxCallback('ajaxComplete', {xhr: xhr}, 'complete', xhr, xhr.status);
  };

  // Ajax start callback
  fireAjaxCallback('ajaxStart', {xhr: xhr}, 'start', xhr);
  fireAjaxCallback(undefined, undefined, 'beforeSend', xhr);

  function sendRequest() {
    var callbackId = window.plus.bridge.callbackId(success, fail);
    var args = [callbackId, options.method, options.url, postData];
    window.plus.bridge.exec(PLUGIN_NAME, "request", args);
  }

  if (window.plus && window.plus.isReady) {
    sendRequest();
  } else {
    document.addEventListener('plusready', function() {
      sendRequest();
    }, false);
  }

  return xhr;
};

// Shrotcuts
(function () {
  var methods = ('get post getJSON').split(' ');
  function createMethod(method) {
    muna[method] = function (url, data, success) {
      return muna.ajax({
          url: url,
          method: method === 'post' ? 'POST' : 'GET',
          data: typeof data === 'function' ? undefined : data,
          success: typeof data === 'function' ? data : success,
          dataType: method === 'getJSON' ? 'json' : undefined
      });
    };
  }
  for (var i = 0; i < methods.length; i++) {
    createMethod(methods[i]);
  }
})();